import React, {useEffect, useRef} from 'react'

//This function will wrap all the logic for the Linechart component
function CirclePlot({node, width, height}){
  const parent = useRef(node)
  console.log("Creating CirclePlot component.", node, width, parent)
  useEffect(() => {
    createCirclePlot()
  })
	return <g ref={parent}></g>

  function createCirclePlot(){
    console.log("creating circle plot", parent)
    const parentSel = d3.select(parent.current)
    const dataSource = 'https://gist.githubusercontent.com/Razpudding/f884509f465766aa4ad31e9cf18a696c/raw/3aa25817113f50f0a0695b87fffc4e234ac1c8d0/eyecolor-gender.json'
    
    const x = d3.scaleLinear()
    const y = d3.scaleLinear()
		
    //Load the data
    d3.json(dataSource)
      .then(data => {
      	//Filter the data
        data = data.filter(validColor)
      	//Transform the data so we get colors
        data = data.map(transformColor)
        data.sort( (a,b) => d3.ascending(a.b, b.b))
      	//Draw the circles in the scatterplot
        drawCircles(data)
      	//Scale the circles
        addScales()
    })

    function drawCircles(data){
      console.log("drawmap called with elements", data.length)
      // console.log(svg.selectAll(".dot"))
      let circles = parentSel.selectAll(".dot")
        .data(data)
        .enter()
        .append("circle")
          .attr("class", "dot")
          .attr("r", 3)
          .attr("cx", d => d.r * 3)
          //We'll add an opacity so circles can't hide behind others
          .attr("opacity", .9)
          //Note that y position is inversed, the more green, the LOWER the position
          .attr("cy", d => d.g)
          .attr("fill", d => d.eyeColor)
    }

    function addScales(){
      x.domain([0,255])
        .range([0, width])
      y.domain([0,255])
        //By putting height first, we're fixing the inverse effect of y
        .range([height,0])

      // console.log(x.range())
      // console.log("scaleTest", x(50))
      // console.log(svg.selectAll(".dot"))
      parentSel.selectAll(".dot")
        .transition()
          .delay(function(d, i) { return i * 20; })
          .attr("cx", d => x(d.r))
          .attr("cy", d => y(d.g))
          .ease(d3.easeBounce)
          .duration(1500)
      	.transition()
      		.attr("r", d => Math.sqrt(d.b * 5))
    }

    //Check if a color is valid with a regex tester
    function validColor(row){
      row.eyeColor = row.eyeColor.toUpperCase()
      return (/^((0x){0,1}|#{0,1})([0-9A-F]{8}|[0-9A-F]{6})$/).test(row.eyeColor) 
    }

    //Adds r,g,b properties
    function transformColor(row){
      if (row.eyeColor != "undefined"){
        row.eyeColor = row.eyeColor.replace("#","")
        row.r = parseInt(row.eyeColor.substring(0,2), 16)
        row.g = parseInt(row.eyeColor.substring(2,4), 16)
        row.b = parseInt(row.eyeColor.substring(4,6), 16)
        row.eyeColor = row.eyeColor.charAt(0) == "#"? row.eyeColor : "#" + row.eyeColor
      }
      // console.log(row)
      return row
    }
	}
}
export default CirclePlot